# -*- coding: utf-8 -*-

from pyelong.api.base import ApiBase


class Data(ApiBase):
    _category = 'hotel'

    def rp(self, **kwargs):
        """
        产品详情，方法名：hotel.data.rp
        http://open.elong.com/wiki/Hotel.data.rp
        NOTE: 本接口属于高级功能接口，通常开发使用不到，不建议使用。
        """
        return self._request('rp', **kwargs)

    def inventory(self, **kwargs):
        """
        房态库存，方法名：hotel.data.inventory
        http://open.elong.com/wiki/Hotel.data.inventory
        NOTE: 本接口属于高级功能接口，通常开发使用不到，不建议使用。
        NOTE: 本接口不能获取万豪集团酒店的库存数据；万豪集团酒店的报价请使用hotel.detail接口。
        """
        return self._request('inventory', **kwargs)

    def rate(self, **kwargs):
        """
        产品详情，方法名：hotel.data.rate
        http://open.elong.com/wiki/Hotel.data.rate
        NOTE: 本接口属于高级功能接口，通常开发使用不到，不建议使用。
        NOTE: 本接口不能获取直连酒店(目前是万豪集团的酒店)的价格数据；其报价请使用hotel.detail接口。
        """
        return self._request('rate', **kwargs)

    def validate(self, **kwargs):
        """
        房态库存，方法名：hotel.data.validate
        http://open.elong.com/wiki/Hotel.data.validate
        """
        return self._request('validate', **kwargs)



# -*- coding: utf-8 -*-

class ApiBase(object):
    _category = ''

    def __init__(self, client):
        self._client = client

    def _request(self, method, https=False, raw=False, **kwargs):
        api = self._build_api(method)
        return self._client.request.do(api, kwargs, https, raw)

    def _build_api(self, method):
        class_name = self.__class__.__name__
        api = '.'.join([self._category, class_name, method]).lower()
        return api.strip('.')



# -*- coding: utf-8 -*-

from pyelong.api.base import ApiBase


class Incr(ApiBase):
    _category = 'hotel'

    def id(self, **kwargs):
        """
        增量编号，方法名：hotel.incr.id
        http://open.elong.com/wiki/Hotel.incr.id
        NOTE: 本接口属于高级功能接口，通常开发使用不到，不建议使用。可以考虑使用订单增量。
        """
        return self._request('id', **kwargs)

    def data(self, **kwargs):
        """
        数据变化增量，方法名：hotel.incr.data
        http://open.elong.com/wiki/Hotel.incr.data
        NOTE: 本接口属于高级功能接口，通常开发使用不到，不建议使用。
        NOTE: 本接口不能获取直连酒店(目前是万豪集团的酒店)的数据的变化；其报价请使用hotel.detail接口。
        """
        return self._request('data', **kwargs)

    def inv(self, **kwargs):
        """
        库存增量，方法名：hotel.incr.inv
        http://open.elong.com/wiki/Hotel.incr.inv
        NOTE: 本接口属于高级功能接口，通常开发使用不到，不建议使用。
        NOTE: 本接口不能获取直连酒店(目前是万豪集团的酒店)的数据的变化；其报价请使用hotel.detail接口。
        """
        return self._request('inv', **kwargs)

    def rate(self, **kwargs):
        """
        价格增量，方法名：hotel.incr.rate
        http://open.elong.com/wiki/Hotel.incr.rate
        NOTE: 本接口属于高级功能接口，通常开发使用不到，不建议使用。
        NOTE: 本接口不能获取直连酒店(目前是万豪集团的酒店)的数据的变化；其报价请使用hotel.detail接口。
        """
        return self._request('rate', **kwargs)

    def order(self, **kwargs):
        """
        订单增量，方法名：hotel.incr.order，必须使用 https
        http://open.elong.com/wiki/Hotel.incr.order
        """
        return self._request('order', https=True, **kwargs)

    def state(self, **kwargs):
        """
        状态增量，方法名：hotel.incr.state
        http://open.elong.com/wiki/Hotel.incr.state
        """
        return self._request('state', **kwargs)



# -*- coding: utf-8 -*-

from pyelong.api.base import ApiBase
from pyelong.api.ihotel.order import Order

__all__ = ('Ihotel')


class Ihotel(ApiBase):
    _category = ''

    def list(self, **kwargs):
        """
        国际酒店列表，方法名：ihotel.list
        http://open.elong.com/wiki/Ihotel.list
        """
        return self._request('list', raw=True, **kwargs)

    def detail(self, **kwargs):
        """
        国际酒店详情，方法名：ihotel.detail
        http://open.elong.com/wiki/Ihotel.detail
        """
        return self._request('detail', raw=True, **kwargs)

    @property
    def order(self):
        return Order(self._client)



# -*- coding: utf-8 -*-

from pyelong.api.base import ApiBase
from pyelong.api.common.creditcard import CreditCard

__all__ = 'Common'


class Common(ApiBase):
    _category = ''

    def exchangerate(self, **kwargs):
        """
        获取外币对应人民币的汇率，方法名：common.exchangerate
        http://open.elong.com/wiki/Common.exchangerate
        """
        return self._request('exchangerate', **kwargs)

    @property
    def creditcard(self):
        return CreditCard(self._client)



#
# -*- coding: utf-8 -*-
#
#  Util/Padding.py :  Functions to manage padding
#
# ===================================================================
# The contents of this file are dedicated to the public domain.  To
# the extent that dedication to the public domain is not available,
# everyone is granted a worldwide, perpetual, royalty-free,
# non-exclusive license to exercise all rights associated with the
# contents of this file for any purpose whatsoever.
# No rights are reserved.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
# ===================================================================

""" Functions to manage padding

This module provides minimal support for adding and removing standard padding
from data.
"""

__all__ = [ 'ValueError', 'pad', 'unpad' ]

from Crypto.Util.py3compat import *


def pad(data_to_pad, block_size, style='pkcs7'):
    """Apply standard padding.

    :Parameters:
      data_to_pad : byte string
        The data that needs to be padded.
      block_size : integer
        The block boundary to use for padding. The output length is guaranteed
        to be a multiple of ``block_size``.
      style : string
        Padding algorithm. It can be *'pkcs7'* (default), *'iso7816'* or *'x923'*.
    :Return:
      The original data with the appropriate padding added at the end.
    """

    padding_len = block_size-len(data_to_pad)%block_size
    if style == 'pkcs7':
        padding = bchr(padding_len)*padding_len
    elif style == 'x923':
        padding = bchr(0)*(padding_len-1) + bchr(padding_len)
    elif style == 'iso7816':
        padding = bchr(128) + bchr(0)*(padding_len-1)
    else:
        raise ValueError("Unknown padding style")
    return data_to_pad + padding

def unpad(padded_data, block_size, style='pkcs7'):
    """Remove standard padding.

    :Parameters:
      padded_data : byte string
        A piece of data with padding that needs to be stripped.
      block_size : integer
        The block boundary to use for padding. The input length
        must be a multiple of ``block_size``.
      style : string
        Padding algorithm. It can be *'pkcs7'* (default), *'iso7816'* or *'x923'*.
    :Return:
        Data without padding.
    :Raises ValueError:
        if the padding is incorrect.
    """

    pdata_len = len(padded_data)
    if pdata_len % block_size:
        raise ValueError("Input data is not padded")
    if style in ('pkcs7', 'x923'):
        padding_len = bord(padded_data[-1])
        if padding_len<1 or padding_len>min(block_size, pdata_len):
            raise ValueError("Padding is incorrect.")
        if style == 'pkcs7':
            if padded_data[-padding_len:]!=bchr(padding_len)*padding_len:
                raise ValueError("PKCS#7 padding is incorrect.")
        else:
            if padded_data[-padding_len:-1]!=bchr(0)*(padding_len-1):
                raise ValueError("ANSI X.923 padding is incorrect.")
    elif style == 'iso7816':
        padding_len = pdata_len - padded_data.rfind(bchr(128))
        if padding_len<1 or padding_len>min(block_size, pdata_len):
            raise ValueError("Padding is incorrect.")
        if padding_len>1 and padded_data[1-padding_len:]!=bchr(0)*(padding_len-1):
            raise ValueError("ISO 7816-4 padding is incorrect.")
    else:
        raise ValueError("Unknown padding style")
    return padded_data[:-padding_len]




# -*- coding: utf-8 -*-

import logging

import tornado.escape

_logger = logging.getLogger(__name__)


class Response(object):
    def __init__(self, resp, content_getter):
        if not callable(content_getter):
            raise ValueError('content_getter must be callable')
        self._resp = resp
        self._content_getter = content_getter
        self.parse_content()

    def parse_content(self):
        """
        详情见：http://open.elong.com/wiki/%E5%B9%B3%E5%8F%B0%E5%8D%8F%E8%AE%AE
        """
        if 200 <= self.status_code < 300:
            #: Elong API 请求返回的结果代码
            #: 0 表示成功完成了请求；有些请求逻辑是否成功需要继续判断 Result
            content = self._content_getter(self._resp)
            self.code = content['Code']

            if '|' in self.code:
                self.code, self.error = self.code.split('|', 1)
            elif content.get('Message'):
                self.error = content.get('Message', None)
            else:
                self.error = None

            self.code = self.code.strip()
            #: Elong API 请求返回的结果数据对象
            self.result = content.get('Result', None)
        else:
            # 请求出现异常
            self.code = str(self.status_code)
            self.reason = self._resp.reason
            self.result = None

        self._log_request()

    @property
    def ok(self):
        return self.code == '0'

    @property
    def url(self):
        return None

    @property
    def status_code(self):
        return 200

    @property
    def headers(self):
        return self._resp.headers

    @property
    def request_time(self):
        """
        :return: milliseconds from request start to finish
        """
        return 0

    def _log_request(self):
        if self.ok:
            _logger.info('pyelong request: %s', self.url)
            _logger.info('pyelong result: %s', self.result)
        else:
            _logger.error('pyelong request error, url: %s, code: %s, error: %s',
                          self.url, self.code, self.error)

        _logger.info('pyelong elapsed: %s ms', self.request_time)

    def to_json(self):
        return {
            'url': self.url,
            'status_code': self.status_code,
            'request_time': self.request_time,
            'code': self.code,
            'error': self.error,
            'result': self.result
        }

    def __str__(self):
        return '%s %s %s' % (self.code, self.error, self.result)


class RequestsResponse(Response):
    def __init__(self, resp):
        super(RequestsResponse, self).__init__(resp, lambda x: x.json())

    @property
    def url(self):
        return self._resp.url

    @property
    def status_code(self):
        return self._resp.status_code

    @property
    def request_time(self):
        return int(self._resp.elapsed.total_seconds() * 1000)


class TornadoResponse(Response):
    def __init__(self, resp):
        content_getter = lambda x: tornado.escape.json_decode(x.body)
        super(TornadoResponse, self).__init__(resp, content_getter)

    @property
    def url(self):
        return self._resp.effective_url

    @property
    def status_code(self):
        return self._resp.code

    @property
    def request_time(self):
        return int(self._resp.request_time * 1000)



# -*- coding: utf-8 -*-

__author__ = 'leon'

from Crypto.Cipher import DES

try:
    from Crypto.Util.Padding import pad, unpad
except ImportError:
    from .padding import pad, unpad


def des_encrypt(data, key, iv, mode=DES.MODE_CBC):
    cipher = DES.new(key, mode=mode, IV=iv)
    padded_text = pad(data, cipher.block_size)
    return cipher.encrypt(padded_text).encode('hex')


def des_decrypt(data, key, iv, mode=DES.MODE_CBC):
    cipher = DES.new(key, mode=mode, IV=iv)
    decrypted = cipher.decrypt(data.decode('hex'))
    return unpad(decrypted, cipher.block_size)



# -*- coding: utf-8 -*-

from pyelong.api.base import ApiBase
from pyelong.api.hotel.order import Order
from pyelong.api.hotel.data import Data
from pyelong.api.hotel.incr import Incr

__all__ = 'Hotel'


class Hotel(ApiBase):
    _category = ''

    def list(self, **kwargs):
        """
        酒店列表搜索，方法名：hotel.list
        http://open.elong.com/wiki/Hotel.list
        """
        return self._request('list', **kwargs)

    def detail(self, **kwargs):
        """
        酒店详情搜索，方法名：hotel.detail
        http://open.elong.com/wiki/Hotel.detail
        """
        return self._request('detail', **kwargs)

    @property
    def order(self):
        return Order(self._client)

    @property
    def data(self):
        return Data(self._client)

    @property
    def incr(self):
        return Incr(self._client)

    @property
    def id(self):
        return ID(self._client)

    @property
    def inv(self):
        return Inv(self._client)


class ID(ApiBase):
    _category = 'hotel'

    def list(self, **kwargs):
        """
        酒店Id列表搜索，方法名： hotel.id.list
        http://open.elong.com/wiki/Hotel.id.list
        """
        return self._request('list', **kwargs)


class Inv(ApiBase):
    _category = 'hotel'

    def validate(self, **kwargs):
        """
        库存验证，方法名：hotel.inv.validate
        http://open.elong.com/wiki/Hotel.inv.validate
        """
        return self._request('validate', **kwargs)



# -*- coding: utf-8 -*-

from pyelong.api.base import ApiBase


class Order(ApiBase):
    _category = 'ihotel'

    def list(self, **kwargs):
        """
        订单列表，方法名称：ihotel.order.list，必须使用 https
        http://open.elong.com/wiki/Ihotel.order.list
        """
        return self._request('list', https=True, raw=True, **kwargs)

    def create(self, **kwargs):
        """
        创建订单，方法名称：ihotel.order.create，必须使用 https
        http://open.elong.com/wiki/Ihotel.order.create
        """
        return self._request('create', https=True, raw=True, **kwargs)

    def detail(self, **kwargs):
        """
        订单详情，方法名称：ihotel.order.detail，必须使用 https
        http://open.elong.com/wiki/Ihotel.order.detail
        """
        return self._request('detail', https=True, raw=True, **kwargs)

    def canceltips(self, **kwargs):
        """
        订单取消提示，方法名称：ihotel.order.canceltips，必须使用 https
        http://open.elong.com/wiki/Ihotel.order.canceltips
        """
        return self._request('canceltips', https=True, raw=True, **kwargs)

    def cancel(self, **kwargs):
        """
        订单取消，方法名称：ihotel.order.cancel，必须使用 https
        http://open.elong.com/wiki/Ihotel.order.cancel
        """
        return self._request('cancel', https=True, raw=True, **kwargs)



# -*- coding: utf-8 -*-

from pyelong.api.base import ApiBase


class CreditCard(ApiBase):
    _category = 'common'

    def validate(self, **kwargs):
        """
        信用卡验证，方法名：common.creditcard.validate，必须使用 https
        http://open.elong.com/wiki/Common.creditcard.validate
        """
        return self._request('validate', https=True, **kwargs)



# -*- coding: utf-8 -*-

import hashlib
import json
import time
import urllib

import requests
from tornado import gen

from tornado.httpclient import AsyncHTTPClient

from pyelong.api import ApiSpec
from pyelong.response import RequestsResponse, TornadoResponse


class Request(object):
    def __init__(self, user, app_key, secret_key,
                 cert=None,
                 host=ApiSpec.host,
                 version=ApiSpec.version,
                 local=ApiSpec.local):
        self.user = user
        self.app_key = app_key
        self.secret_key = secret_key

        self.cert = cert
        self.verify_ssl = self.cert is not None

        self.host = host
        self.version = version
        self.local = local

    def do(self, api, params, https, raw=False):
        raise NotImplementedError()

    def prepare(self, api, params, https, raw):
        timestamp = str(int(time.time()))
        data = self.build_data(params, raw)
        scheme = 'https' if https else 'http'
        url = "%s://%s" % (scheme, self.host)
        params = {
            'method': api,
            'user': self.user,
            'timestamp': timestamp,
            'data': data,
            'signature': self.signature(data, timestamp),
            'format': 'json'
        }
        return url, params

    def build_data(self, params, raw=False):
        if not raw:
            data = {
                'Version': self.version,
                'Local': self.local,
                'Request': params
            }
        else:
            data = params
        return json.dumps(data, separators=(',', ':'))

    def signature(self, data, timestamp):
        s = self._md5(data + self.app_key)
        return self._md5("%s%s%s" % (timestamp, s, self.secret_key))

    @staticmethod
    def _md5(data):
        return hashlib.md5(data.encode('utf-8')).hexdigest()


class SyncRequest(Request):
    @property
    def session(self):
        if not hasattr(self, '_session') or not self._session:
            self._session = requests.Session()
        return self._session

    def do(self, api, params, https, raw=False):
        url, params = self.prepare(api, params, https, raw)
        result = self.session.get(url, params=params,
                                  verify=self.verify_ssl, cert=self.cert)
        resp = RequestsResponse(result)
        return resp


class AsyncRequest(Request):
    @staticmethod
    def _encode_params(data):
        """
        :param dict data: params

        token from requests.models.RequestEncodingMixin._encode_params
        """
        result = []
        for k, vs in data.iteritems():
            if isinstance(vs, basestring) or not hasattr(vs, '__iter__'):
                vs = [vs]
            for v in vs:
                if v is not None:
                    result.append(
                        (k.encode('utf-8') if isinstance(k, str) else k,
                         v.encode('utf-8') if isinstance(v, str) else v))
        return urllib.urlencode(result, doseq=True)

    def _prepare_url(self, url, params):
        if url.endswith('/'):
            url = url.strip('/')
        return '%s?%s' % (url, self._encode_params(params))

    @gen.coroutine
    def do(self, api, params, https, raw=False):
        url, params = self.prepare(api, params, https, raw)
        # use the default SimpleAsyncHTTPClient
        resp = yield AsyncHTTPClient().fetch(self._prepare_url(url, params),
                                             validate_cert=self.verify_ssl,
                                             ca_certs=self.cert)
        resp = TornadoResponse(resp)
        raise gen.Return(resp)



# -*- coding: utf-8 -*-

from pyelong.api.base import ApiBase


class Order(ApiBase):
    _category = 'hotel'

    def create(self, **kwargs):
        """
        创建订单，方法名：hotel.order.create，必须使用 https
        http://open.elong.com/wiki/Hotel.order.create
        """
        return self._request('create', https=True, **kwargs)

    def list(self, **kwargs):
        """
        订单列表，方法名：hotel.order.list，必须使用 https
        http://open.elong.com/wiki/Hotel.order.list
        """
        return self._request('list', https=True, **kwargs)

    def detail(self, **kwargs):
        """
         订单详情，方法名：hotel.order.create，必须使用 https
         http://open.elong.com/wiki/Hotel.order.create
        """
        return self._request('detail', https=True, **kwargs)

    def checkguest(self, **kwargs):
        """
        客人姓名验证，方法名：hotel.order.checkguest
        http://open.elong.com/wiki/Hotel.order.checkguest
        """
        return self._request('checkguest', https=True, **kwargs)

    def update(self, **kwargs):
        """
        修改订单，方法名：hotel.order.update，必须使用 https
        http://open.elong.com/wiki/Hotel.order.update
        """
        return self._request('update', https=True, **kwargs)

    def cancel(self, **kwargs):
        """
        取消订单，方法名：hotel.order.cancel，必须使用 https
        http://open.elong.com/wiki/Hotel.order.cancel
        """
        return self._request('cancel', https=True, **kwargs)

    def instant(self, **kwargs):
        """
        即时确认，方法名：hotel.order.instant，必须使用 https
        http://open.elong.com/wiki/Hotel.order.instant
        """
        return self._request('instant', https=True, **kwargs)

    def related(self, **kwargs):
        """
        关联订单，方法名：hotel.order.related，必须使用 https
        http://open.elong.com/wiki/Hotel.order.related
        """
        return self._request('related', https=True, **kwargs)

    def recent(self, **kwargs):
        """
        最近预订，方法名：hotel.order.recent，必须使用 https
        http://open.elong.com/wiki/Hotel.order.recent
        """
        return self._request('recent', https=True, **kwargs)

    def feedback(self, **kwargs):
        """
        入住反馈，方法名：hotel.order.feedback，必须使用 https
        http://open.elong.com/wiki/Hotel.order.feedback
        """
        return self._request('feedback', https=True, **kwargs)

    def promote(self, **kwargs):
        """
        订单催确认，方法名：hotel.order.promote，必须使用 https
        http://open.elong.com/wiki/Hotel.order.promote
        """
        return self._request('promote', https=True, **kwargs)



#!/usr/bin/env python

import re
from setuptools import setup

with open('pyelong/__init__.py', 'r') as fd:
    version = re.search(r'^__version__\s*=\s*[\'"]([^\'"]*)[\'"]',
                        fd.read(), re.MULTILINE).group(1)

setup(
    name='pyelong',
    version=version,
    packages=[
        'pyelong',
        'pyelong.api',
        'pyelong.api.hotel',
        'pyelong.api.ihotel',
        'pyelong.api.common',
        'pyelong.util'
    ],
    url='https://github.com/DeanThompson/pyelong',
    license='MIT',
    author='Yangliang Li',
    author_email='yanglianglee@gmail.com',
    description='Python SDK for Elong (http://elong.com/) rest APIs.',
    zip_safe=False,
    install_requires=[
        'requests>=2.7.0',
        'tornado>=4.2',
        'pycrypto>=2.6.1'
    ]
)



# -*- coding: utf-8 -*-

import time

from pyelong.request import SyncRequest, AsyncRequest
from pyelong.api.hotel import Hotel
from pyelong.api.ihotel import Ihotel
from pyelong.api.common import Common
from pyelong.util import des_encrypt


class Client(object):
    def __init__(self, user, app_key, secret_key,
                 use_tornado=False, cert=None, **kwargs):
        """ 初始化一个客户端对象
        :param str user: API 账号，需要在艺龙注册后得到
        :param str app_key: API 调用的身份标识，需要在艺龙注册后得到
        :param str secret_key: API 调用的密钥，用于签名，需要在艺龙注册后得到
        :param bool use_tornado: 默认使用 requests，如果为 True 使用 AsyncHTTPClient
        :param str cert: SSL 证书文件路径，如果不为 None，则会检查服务器的证书
        :param kwargs: 可选参数：
                    - host: 指定 API 的 host，默认是：api.elong.com/rest
                    - version: 指定 API 版本
                    - local: 指定语言，默认是 zh_CN
        """
        request_class = AsyncRequest if use_tornado else SyncRequest
        self.request = request_class(user, app_key, secret_key, cert, **kwargs)

    @property
    def hotel(self):
        return Hotel(self)

    @property
    def ihotel(self):
        return Ihotel(self)

    @property
    def common(self):
        return Common(self)

    def encrypt_credit_card_field(self, value, timestamp=None):
        """ 加密信用卡信息
        :param value: 字段值
        :param timestamp: 时间戳
        :return: 16 进制数
        """
        key = self.request.app_key[-8:]
        if not timestamp:
            timestamp = str(int(time.time()))
        data = "%s#%s" % (timestamp, value)
        return des_encrypt(data, key=key, iv=key)
